@@ -41,15 +41,15 @@ PYLIBSSH2_Channel_close(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
     int rc;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_close(self->channel);
-    MY_END_ALLOW_THREADS(self->tstate);
+    rc = libssh2_channel_wait_closed(self->channel);
+    Py_END_ALLOW_THREADS
 
     if (rc) {
         /* CLEAN: PYLIBSSH2_CHANNEL_CANT_CLOSE_MSG */
         PyErr_SetString(PYLIBSSH2_Error, "Unable to close the channel.");
-        Py_INCREF(Py_None);
-        return Py_None; 
+        return NULL; 
     }
 
     return Py_BuildValue("i", rc);
@@ -94,26 +94,25 @@ PYLIBSSH2_Channel_pty(PYLIBSSH2_CHANNEL *self, PyObject *args)
         return NULL;
     }
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_request_pty_ex(self->channel, term, term_len, modes, modes_len,
                                         width, height, width_px, height_px);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc) {
         /* CLEAN: PYLIBSSH2_CHANNEL_PTY_FAILED_MSG */ 
         PyErr_SetString(PYLIBSSH2_Error, "Failed to request pty.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
-} 
+}
 /* }}} */
 
 /* {{{ PYLIBSSH2_Channel_pty_resize
  */
 static char PYLIBSSH2_Channel_pty_resize_doc[] = "\n\
-pty_resize(width, height) -> int\n\
+pty_resize(width, height, width_px, height_px) -> int\n\
 \n\
 Requests a pty resize on a channel with the given width and height.\n\
 \n\
@@ -121,6 +120,10 @@ Requests a pty resize on a channel with the given width and height.\n\
 @type   width: int\n\
 @param  height: terminal height\n\
 @type   height: int\n\
+@param  width_px: terminal width in pixel (opt)\n\
+@type   height: int\n\
+@param  height_px: terminal height in pixel (opt)\n\
+@type   height: int\n\
 \n\
 @return 0 on success or negative on failure\n\
 @rtype  int";
@@ -141,23 +144,19 @@ PYLIBSSH2_Channel_pty_resize(PYLIBSSH2_CHANNEL *self, PyObject *args)
     int width_px  = 0;
     int height_px = 0;
 
-    if (!PyArg_ParseTuple(args,"ii|ii:pty_resize", &width,&height,
-                                                    &width_px,&height_px)){
+    if (!PyArg_ParseTuple(args,"ii|ii:pty_resize", &width, &height,
+                                                   &width_px, &height_px)){
         return NULL;
     }
-    /* FIXME: For unknown reason, the activation of the two following methods
-     * provokes a core dump of the lib with the following error:
-     * PyEval_RestoreThread: NULL tstate
-     */
-    /*MY_BEGIN_ALLOW_THREADS(self->tstate);*/
+
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_request_pty_size_ex(self->channel, width, height,
                                                 width_px, height_px);
-    /*MY_END_ALLOW_THREADS(self->tstate);*/
+    Py_END_ALLOW_THREADS
 
     if (rc) {
         PyErr_SetString(PYLIBSSH2_Error, "Failed to resize pty");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i",rc);
@@ -179,15 +178,14 @@ PYLIBSSH2_Channel_shell(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
     int rc;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_shell(self->channel);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc) {
         /* CLEAN: PYLIBSSH2_CHANNEL_CANT_REQUEST_SHELL_MSG */
         PyErr_SetString(PYLIBSSH2_Error,"Unable to request shell on allocated pty.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
@@ -217,15 +215,14 @@ PYLIBSSH2_Channel_execute(PYLIBSSH2_CHANNEL *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "s:execute", &command))
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_exec(self->channel, command);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc) {
         /* CLEAN: PYLIBSSH2_CANT_REQUEST_EXEC_COMMAND_MSG */
         PyErr_SetString(PYLIBSSH2_Error, "Unable to request exec command.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
@@ -259,15 +256,14 @@ PYLIBSSH2_Channel_setenv(PYLIBSSH2_CHANNEL *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "ss:setenv", &env_key, &env_val))
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_setenv(self->channel, env_key, env_val);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc == -1) {
         /* CLEAN: PYLIBSSH2_CANT_SET_ENVRIONNEMENT_VARIABLE_MSG */
         PyErr_SetString(PYLIBSSH2_Error,"Unable to set envrionnement variable.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
@@ -294,8 +290,8 @@ PYLIBSSH2_Channel_setblocking(PYLIBSSH2_CHANNEL *self, PyObject *args)
         return NULL;
 
     libssh2_channel_set_blocking(self->channel, block);
-    Py_INCREF(Py_None);
 
+    Py_INCREF(Py_None);
     return Py_None;
 }
 /* }}} */
@@ -330,10 +326,10 @@ PYLIBSSH2_Channel_read(PYLIBSSH2_CHANNEL *self, PyObject *args)
     }
 
     if (libssh2_channel_eof(self->channel) != 1) {
-        MY_BEGIN_ALLOW_THREADS(self->tstate);
+        Py_BEGIN_ALLOW_THREADS
         rc = libssh2_channel_read(self->channel, PyString_AsString(buffer),
                                   buffer_size);
-        MY_END_ALLOW_THREADS(self->tstate);
+        Py_END_ALLOW_THREADS
 
         if (rc > 0) {
             if (rc != buffer_size && _PyString_Resize(&buffer, rc) < 0)
@@ -349,6 +345,55 @@ PYLIBSSH2_Channel_read(PYLIBSSH2_CHANNEL *self, PyObject *args)
 }
 /* }}} */
 
+/* {{{ PYLIBSSH2_Channel_read_ex
+ */
+static char PYLIBSSH2_Channel_read_ex_doc[] = "\n\
+read(size, [stream_id]) -> str\n\
+\n\
+Reads size bytes on the channel.\n\
+\n\
+@param size: size of the buffer storage\n\
+@type  size: int\n\
+@param stream_id: substream ID number\n\
+@type  stream_id: int\n\
+\n\
+@return string containing bytes read or negative value on failure\n\
+@rtype  str or int";
+
+static PyObject *
+PYLIBSSH2_Channel_read_ex(PYLIBSSH2_CHANNEL *self, PyObject *args)
+{
+    int rc;
+    int buffer_size;
+    int stream_id = 0;
+    /* buffer to read as a python object */
+    PyObject *buffer;
+    char * cbuf;
+
+    if (!PyArg_ParseTuple(args, "i|i:read_ex", &buffer_size, &stream_id))
+        return NULL;
+    buffer = PyString_FromStringAndSize(NULL, buffer_size);
+    if (buffer == NULL) {
+        return NULL;
+    }
+    cbuf = PyString_AsString(buffer);
+
+    Py_BEGIN_ALLOW_THREADS
+    rc = libssh2_channel_read_ex(self->channel, stream_id, cbuf, buffer_size);
+    Py_END_ALLOW_THREADS
+
+    if (rc > 0) {
+       if (rc != buffer_size && _PyString_Resize(&buffer, rc) < 0)
+            return NULL;
+    }
+    /**
+       we do NOT increment the reference on the buffer anymore
+       it is already done by PyString_FromStringAndSize
+    */
+    return Py_BuildValue("iN", rc, buffer);
+}
+/* }}} */
+
 /* {{{ PYLIBSSH2_Channel_write
  */
 static char PYLIBSSH2_Channel_write_doc[] = "\n\
@@ -372,15 +417,14 @@ PYLIBSSH2_Channel_write(PYLIBSSH2_CHANNEL *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "s#:write", &message, &message_len))
         return NULL;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_write(self->channel, message, message_len);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc == -1) {
         /* CLEAN: PYLIBSSH2_CANT_WRITE_CHANNEL_MSG */
         PyErr_SetString(PYLIBSSH2_Error,"Unable to write channel.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
@@ -402,15 +446,14 @@ PYLIBSSH2_Channel_flush(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
     int rc;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_flush(self->channel);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc == -1) {
         /* CLEAN: PYLIBSSH2_CANT_FLUSH_CHANNEL_MSG */
         PyErr_SetString(PYLIBSSH2_Error, "Unable to flush channel.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
@@ -466,21 +509,46 @@ PYLIBSSH2_Channel_send_eof(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
     int rc;
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_send_eof(self->channel);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
     if (rc == -1) {
         /* CLEAN: PYLIBSSH2_CANT_SEND_EOF_MSG */
         PyErr_SetString(PYLIBSSH2_Error, "Unable to send a EOF on channel.");
-        Py_INCREF(Py_None);
-        return Py_None;
+        return NULL;
     }
 
     return Py_BuildValue("i", rc);
 }
 /* }}} */
 
+/* {{{ PYLIBSSH2_Channel_wait_closed
+ */
+static char PYLIBSSH2_Channel_wait_closed_doc[] = "\n\
+wait_closed() -> int\n\
+\n\
+Wait for the remote channel to ack channel close.\n\
+\n\
+@param channel\n\
+@type libssh2.Channel\n\
+\n\
+@return 0 on success or negative on failure\n\
+@rtype int";
+
+static PyObject *
+PYLIBSSH2_Channel_wait_closed(PYLIBSSH2_CHANNEL *self, PyObject *args)
+{
+    int rc;
+
+    Py_BEGIN_ALLOW_THREADS
+    rc = libssh2_channel_wait_closed(self->channel);
+    Py_END_ALLOW_THREADS
+
+    return Py_BuildValue("i", rc);
+}
+/* }}} */
+
 /* {{{ PYLIBSSH2_Channel_window_read
  */
 static char PYLIBSSH2_Channel_window_read_doc[] = "\n\
@@ -488,82 +556,67 @@ window_read(read_avail, window_size_initial) -> int\n\
 \n\
 Checks the status of the read window.\n\
 \n\
-@param  read_avail: \n\
-@type   read_avail: int\n\
-@param  window_size_initial: \n\
-@type   window_size_initial: int\n\
 \n\
-@return the number of bytes which the remote end may send without overflowing\n\
-the window limit\n\
-@rtype  int";
+@return windows: the number of bytes which the remote end may send without\n\
+@return read_avail: number of bytes available to be read\n\
+@return window_size_initial: as defined by the channel_open request\n\
+@rtype  (int, int, int)";
 
 static PyObject *
 PYLIBSSH2_Channel_window_read(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
-    unsigned long rc;
+    unsigned long rc=0;
     unsigned long read_avail;
     unsigned long window_size_initial;
 
-    if (!PyArg_ParseTuple(args, "|ii:window_read", &read_avail, &window_size_initial)) {
-        return NULL;
-    }
-
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_channel_window_read_ex(self->channel, &read_avail, &window_size_initial);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS
 
-    return PyInt_FromLong(rc);
+    return Py_BuildValue("(kkk)", rc, read_avail, window_size_initial);
 }
 /* }}} */
 
-/* {{{ PYLIBSSH2_Channel_poll
- */
-static char PYLIBSSH2_Channel_poll_doc[] = "\n\
-poll(timeout, [nfds]) -> int\n\
-\n\
-Polls for activity on the channel.\n\
-\n\
-@param  timeout: remaining timeout\n\
-@type   timeout: int\n\
-@param  nfds: number of fds to poll\n\
-@type   nfds: int\n\
-\n\
-@return number of fds with interesting events or negative on failure\n\
-@rtype  int";
+static char PYLIBSSH2_Channel_window_write_doc[] = "\n"
+"window_write(write_avail) -> int\n"
+"\n"
+"check the status of the write window.\n"
+"\n"
+"\n"
+"@return windows: the number of bytes which may be safely writen on the\n"
+"channel without blocking."
+"@return window_size_initial: as defined by the channel_open request\n"
+"@rtype  (int, int)";
 
 static PyObject *
-PYLIBSSH2_Channel_poll(PYLIBSSH2_CHANNEL *self, PyObject *args)
+PYLIBSSH2_Channel_window_write(PYLIBSSH2_CHANNEL *self)
 {
-    int rc;
-    int nfds = 1;
-    long timeout = 0;
-    LIBSSH2_POLLFD fds;
+    unsigned long rc=0;
+    unsigned long window_size_initial;
 
-    if (!PyArg_ParseTuple(args, "i|i:poll", &timeout, &nfds)) {
-        return NULL;
-    }
+    Py_BEGIN_ALLOW_THREADS
+    rc = libssh2_channel_window_write_ex(self->channel, &window_size_initial);
+    Py_END_ALLOW_THREADS
 
-    fds.type = LIBSSH2_POLLFD_CHANNEL;
-    fds.fd.channel = self->channel; /*see PYLIBSSH2_CHANNEL struct*/
-    fds.events  = LIBSSH2_POLLFD_POLLIN;
-
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
-    rc = libssh2_poll(&fds, nfds, timeout);
-    MY_END_ALLOW_THREADS(self->tstate);
-
-    return Py_BuildValue("i", rc);
+    return Py_BuildValue("(kk)", rc, window_size_initial);
 }
 /* }}} */
 
 /* {{{ PYLIBSSH2_Channel_x11_req
  */
 static char PYLIBSSH2_Channel_x11_req_doc[] = "\n\
-x11_req([display]) -> int\n\
+x11_req([single_connection, auth_proto, auth_cookie, display]) -> int\n\
 \n\
 Requests an X11 Forwarding on the channel.\n\
 \n\
+@param  single_connection: forward a single connection\n\
+@type  single_connection: int\n\
+@param  auth_proto: X11 authentication protocol to use\n\
+@type  auth_proto: str\n\
+@param  auth_cookie: the cookie\n\
+@type  auth_cookie: str\n\
 @param  display: screen number\n\
-@param  display: int\n\
+@type  display: int\n\
 \n\
 @return 0 on success or negative on failure\n\
 @rtype  int";
@@ -572,15 +625,18 @@ static PyObject *
 PYLIBSSH2_Channel_x11_req(PYLIBSSH2_CHANNEL *self, PyObject *args)
 {
     int rc;
+    int single_connection = 0;
+    char *auth_proto = NULL;
+    char *auth_cookie = NULL;
     int display = 0;
 
-    if (!PyArg_ParseTuple(args, "|i:poll_x11_req", &display)) {
+    if (!PyArg_ParseTuple(args, "|issi:x11_req", &single_connection, &auth_proto, &auth_cookie, &display)) {
         return NULL;
     }
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
-    rc = libssh2_channel_x11_req(self->channel, display);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
+    rc = libssh2_channel_x11_req_ex(self->channel, single_connection, auth_proto, auth_cookie, display);
+    Py_END_ALLOW_THREADS
 
     return Py_BuildValue("i", rc);
 }
@@ -609,9 +665,9 @@ PYLIBSSH2_Channel_poll_read(PYLIBSSH2_CHANNEL *self, PyObject *args)
         return NULL;
     }
 
-    MY_BEGIN_ALLOW_THREADS(self->tstate);
+    Py_BEGIN_ALLOW_THREADS
     rc = libssh2_poll_channel_read(self->channel, extended);
-    MY_END_ALLOW_THREADS(self->tstate);
+    Py_END_ALLOW_THREADS;
 
     return Py_BuildValue("i", rc);
 }
@@ -626,7 +682,7 @@ PYLIBSSH2_Channel_poll_read(PYLIBSSH2_CHANNEL *self, PyObject *args)
 
 #define ADD_METHOD(name) \
 { #name, (PyCFunction)PYLIBSSH2_Channel_##name, METH_VARARGS, PYLIBSSH2_Channel_##name##_doc }
-static PyMethodDef PYLIBSSH2_Channel_methods[] = 
+static PyMethodDef PYLIBSSH2_Channel_methods[] =
 {
     ADD_METHOD(close),
     ADD_METHOD(pty),
@@ -635,16 +691,18 @@ static PyMethodDef PYLIBSSH2_Channel_methods[] =
     ADD_METHOD(execute),
     ADD_METHOD(setenv),
     ADD_METHOD(setblocking),
+    ADD_METHOD(read_ex),
     ADD_METHOD(read),
     ADD_METHOD(write),
     ADD_METHOD(flush),
     ADD_METHOD(eof),
-    ADD_METHOD(send_eof),
-    ADD_METHOD(window_read),
-    ADD_METHOD(poll_read),
-    ADD_METHOD(poll),
-    ADD_METHOD(x11_req),
     ADD_METHOD(exit_status),
+    ADD_METHOD(send_eof),
+    ADD_METHOD(wait_closed),
+    ADD_METHOD(window_read),
+    ADD_METHOD(window_write),
+    ADD_METHOD(poll_read),
+    ADD_METHOD(x11_req),
     { NULL, NULL }
 };
 #undef ADD_METHOD
@@ -663,7 +721,6 @@ PYLIBSSH2_Channel_New(LIBSSH2_CHANNEL *channel, int dealloc)
     }
 
     self->channel = channel;
-    self->tstate = NULL;
     self->dealloc = dealloc;
 
     return self;
@@ -725,7 +782,7 @@ init_libssh2_Channel(PyObject *dict)
     PYLIBSSH2_Channel_Type.ob_type = &PyType_Type;
     Py_XINCREF(&PYLIBSSH2_Channel_Type);
     PyDict_SetItemString(dict, "ChannelType", (PyObject *)&PYLIBSSH2_Channel_Type);
-     
+
     return 1;
 }
 /* }}} */
